       subroutine build(alpha,degs,degsb,edge,first,maxc,
     +                 ne,n2e,n2v,next,
     +                 nS,nSbar,nv,objf,rcl,
     +                 S,Sbar,seed,sums,sumst,
     +                 sumsb,sumsbt,vertex,v,
     +                 vtx1,vtx2,w,ws,wsind)
c     ------------------------------------------------------------------
c
c     build: GRASP construction phase (phase 1)
c
c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          alpha  - GRASP RCL parameter
c          maxc   - upper bound on maxcut cost
c          ne
c          n2e    - array dimension = 2*ne
c          n2v    - array dimension = 2*nv
c          nv     - number of nodes in reduced graph
c
c     ------------------------------------------------------------------
      implicit   none
      real       alpha
      integer    maxc,ne,n2e,n2v,nv

c     ------------------------------------------------------------------
c
c     Passed input/output scalars:
c
c          nS
c          nSbar
c          seed   - seed for pseudo random number generator
c
c     ------------------------------------------------------------------
      integer nS,nSbar,seed

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          objf   - value of objective function
c
c     ------------------------------------------------------------------
      integer objf

c     ------------------------------------------------------------------
c
c     Passed input arrays:
c
c          edge   - edge
c          first  - pointer to start of in-adjacency list (copy)
c          next   - pointer to next element of in-adjacency
c                   list (copy)
c          vertex - existence of vertex in the reduced graph
c          v      - vertex in in-adjacency list (copy)
c          vtx1
c          vtx2
c          w
c          ws
c          wsind
c
c     ------------------------------------------------------------------
      integer    edge(n2e),first(nv),
     +           next(n2e),
     +           vertex(nv),v(n2e),
     +           vtx1(ne),vtx2(ne),w(ne),
     +           ws(ne),wsind(ne)

c     ------------------------------------------------------------------
c
c     Passed output arrays:
c
c          degs
c          degsb
c          S
c          Sbar
c          sums
c          sumsb
c
c     ------------------------------------------------------------------
      integer    degs(nv),degsb(nv),S(nv),Sbar(nv),
     +           sums(nv),sumsb(nv) 

c     ------------------------------------------------------------------
c
c     Passed working arrays:
c
c          rcl    - Restricted Candidate List
c          sumst  -
c          sumsbt -
c
c     ------------------------------------------------------------------
      integer rcl(n2v),sumst(nv),sumsbt(nv)

c     ------------------------------------------------------------------
c
c     Local scalars:
c
c          e      - arc
c          i      - do loop index
c          node   - vertex
c          nrcl   - number of elements to be chosen from
c          nselct - selected edge index
c          nvleft - number of unselected vertices
c          randp  - random number generator
c          xrand  - random number
c
c     ------------------------------------------------------------------
      integer    e,i,node,nrcl,nselct,nvleft
      real       randp,xrand

c     ------------------------------------------------------------------
c     Initialize number of vertices left (nvleft),
c     number of vetices in S (nS) and number of
c     vertices in Sbar (nSbar).
c     ------------------------------------------------------------------
      objf=0
      nvleft=nv-2
      nS=0
      nSbar=0

      do 5 i=1,nv
           sums(i)=0
           sumsb(i)=0
5     continue

      nrcl=ifix(ne*(1.-alpha))+1
      xrand=randp(seed)
      if (nrcl.gt.ne) nrcl=ne
      nselct=1+seed/(2147483647/nrcl)
      nselct=ne-nselct+1
      e=wsind(nselct)

      node=vtx1(e)
      nS=nS+1
      S(nS)=node
      vertex(node)=1
      call upddeg( edge,first,ne,n2e,next,
     +             node,nv,sums,sumsb,v,vertex,w)

      node=vtx2(e)
      nSbar=nSbar+1
      Sbar(nSbar)=node
      vertex(node)=2
      call upddeg( edge,first,ne,n2e,next,
     +             node,nv,sums,sumsb,v,vertex,w)


      objf=objf+ws(nselct)
c     ------------------------------------------------------------------
c     Begin construction loop
c     ------------------------------------------------------------------
10    continue
	   if (nvleft.eq.0) return
c          -------------------------------------------------------------
c          If there are still vertices to be considered,
c          build restricted candidate list (RCL).
c          -------------------------------------------------------------
           call mkrcl(alpha,edge,first,maxc,
     +                ne,n2e,n2v,next,nrcl,nv,rcl,sumst,
     +                sumsbt,vertex,v,w)

c          -------------------------------------------------------------
c          Pick a candidate from the RCL at random and place node into
c          set S or Sbar.
c          -------------------------------------------------------------
           call pickv(edge,first,
     +                ne,n2e,n2v,next,node,nrcl,nS,
     +                nSbar,nv,
     +                nvleft,objf,rcl,S,Sbar,seed,v,vertex,w)

	   call upddeg( edge,first,ne,n2e,next,
     +                  node,nv,sums,sumsb,v,vertex,w )

c          -------------------------------------------------------------
c          Check if there still are vertices to be removed.
c          -------------------------------------------------------------
	   goto 10

c     continue
c     ------------------------------------------------------------------
 
c     ------------------------------------------------------------------
c     End of subroutine build.
c     ------------------------------------------------------------------
c
      end
