      subroutine buildr(nv,n2e,ne,seed,nS,S,nSbar,Sbar,vertex,objf,
     +                      first0,next0,v0,edge0,w,sums,sumsb)
c     ------------------------------------------------------------------
c
c     buildr: random construction (phase 1)
c
c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          n2e    - array dimension = 2*ne
c          nv     - number of nodes in reduced graph
c          ne     - number of nodes in reduced graph
c
c     ------------------------------------------------------------------
      implicit   none
      integer    ne,n2e,nv

c     ------------------------------------------------------------------
c
c     Passed input/output scalars:
c
c          nS
c          nSbar
c          seed   - seed for pseudo random number generator
c
c     ------------------------------------------------------------------
      integer nS,nSbar,seed

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          objf   - value of objective function
c
c     ------------------------------------------------------------------
      integer objf

c     ------------------------------------------------------------------
c
c     Passed input arrays:
c
c          edge0   - edge
c          first0  - pointer to start of in-adjacency list
c          next0   - pointer to next element of in-adjacency
c                   list 
c          w
c          v0      - vertex in adjacency list
c
c     ------------------------------------------------------------------
      integer    edge0(n2e),first0(nv),
     +           next0(n2e),
     +           v0(n2e),
     +           w(ne)

c     ------------------------------------------------------------------
c
c     Passed output arrays:
c
c          S
c          Sbar
c          sums
c          sumsb
c          vertex  - existence of vertex in the reduced graph
c
c     ------------------------------------------------------------------
      integer    S(nv),Sbar(nv),
     +           sums(nv),sumsb(nv),
     +           vertex(nv)

c     ------------------------------------------------------------------
c
c     Local scalars:
c
c          e      - arc
c          i      - do loop index
c          node   - vertex
c          randp  - random number generator
c          ptr    - pointer
c
c     ------------------------------------------------------------------
      integer    e,i,node,ptr
      real       randp

c     ------------------------------------------------------------------
c     Assign vertices to S or Sbar at random.
c     ------------------------------------------------------------------


      nS=0
      nSbar=0
      do 10 i=1,nv
            if (randp(seed) .gt. 0.5) then
		 nS=nS+1
		 S(nS)=i
		 vertex(i)=1
            else
		 nSbar=nSbar+1
		 Sbar(nSbar)=i
		 vertex(i)=2
            endif
10    continue
 
c     ------------------------------------------------------------------
c     Compute sums, sumsb, and objf.
c     ------------------------------------------------------------------
      objf=0
      do 5 i=1,nv
           sums(i)=0
           sumsb(i)=0
	   ptr=first0(i)
30         if (ptr.eq.0) go to 20
	        node=v0(ptr)
		e=edge0(ptr)
		if (vertex(node).eq.1) then
		     sums(i)=sums(i)+w(e)
                else
		     sumsb(i)=sumsb(i)+w(e)
		     if (vertex(i).eq.1) objf=objf+w(e)
                endif
		ptr=next0(ptr)
		goto 30
20         continue
5     continue

c     ------------------------------------------------------------------
c     End of subroutine build.
c     ------------------------------------------------------------------
c
      end
