      subroutine chkgct(errcnd,hammd,look4,maxe,maxitr,maxv,
     +                   ne,nv,prttyp,seed,
     +                   seedd,vtx1,vtx2)
c     ------------------------------------------------------------------
c
c     chkfvs: Checks values of parameters and problem dimension for
c             validity.  Case of feedback vertex set.
c
c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          hammd  - 
c          look4  - size of cutset sought
c          maxe   - array dimension
c          maxitr - maximum number of GRASP iterations
c          maxv   - array dimension
c          ne     - number of arcs in original graph
c          nv     - number of vertices in original graph
c          prttyp - iteration summary report
c                   = 0 - silent
c                   = 1 - show improvements only
c                   = 2 - show all iterations
c          seedd  - seed for pseudo random number generator
c
c     ------------------------------------------------------------------
      implicit none
      integer look4,maxe,maxitr,maxv,ne,nv,prttyp
      double precision seedd
      real hammd

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          errcnd - error condition     
c          seed   - seed for pseudo random number generator
c
c     ------------------------------------------------------------------
      integer errcnd,seed

c     ------------------------------------------------------------------
c
c     Passed input arrays:
c
c          vtx1  - vertex 1 of arc
c          vtx2  - vertex 2 of arc
c
c     ------------------------------------------------------------------
      integer vtx1(ne),vtx2(ne)

c     ------------------------------------------------------------------
c
c     Local scalars:
c
c          i      - do loop index
c
c     ------------------------------------------------------------------
      integer i
c     ------------------------------------------------------------------
c     Check if the number of vertices of the input graph exceeds maxv.
c     ------------------------------------------------------------------
      if (nv.gt.maxv) then
           errcnd=1
           return
      endif

c     ------------------------------------------------------------------
c     Check if the number of arcs of the input graph exceeds maxe.
c     ------------------------------------------------------------------
      if (ne.gt.maxe) then
           errcnd=2
           return
      endif

c     ------------------------------------------------------------------
c     Check if the vertices indices are well-defined and 
c           if the arc weights are well-defined.
c     ------------------------------------------------------------------
      do 10 i=1,ne
           if (vtx1(i).lt.1 .or. vtx1(i).gt.nv) then
                errcnd=3
                return
           endif
           if (vtx2(i).lt.1 .or. vtx2(i).gt.nv) then
                errcnd=3
                return
           endif
10    continue

c     ------------------------------------------------------------------
c     Check if the stopping parameter look4 is well-defined.
c     ------------------------------------------------------------------
      if (look4.lt.0) then
           errcnd=4
           return
      endif

c     ------------------------------------------------------------------
c     Check if the maximum number of GRASP iteration is well-defined.
c     ------------------------------------------------------------------
      if (maxitr.lt.1) then
           errcnd=5
           return
      endif

c     ------------------------------------------------------------------
c     Check if the output option parameter is well-defined
c     ------------------------------------------------------------------
      if ((prttyp.ne.0) .and. (prttyp.ne.1) .and. (prttyp.ne.2) ) then
           errcnd=6
           return
      endif

c     ------------------------------------------------------------------
c     Check if the pseudo random number generator seed is well-defined
c     ------------------------------------------------------------------
      if (seedd.lt.1 .or. seedd.gt.2147483647) then
           errcnd=7
           return
      else
           seed = idint(seedd)
      endif

c     ------------------------------------------------------------------
c     Check if hammd is greater than 0 and less than 1.
c     ------------------------------------------------------------------
      if (hammd.gt.1. .or. hammd .lt. 0.) then
           errcnd=8
           return
      endif
c     ------------------------------------------------------------------
      return
c
c     ------------------------------------------------------------------
c     End of subroutine chkgct.
c     ------------------------------------------------------------------
c
      end
