      subroutine local(edge,first,ne,n2e,next,nS,
     +                 nSbar,nv,objf,S,Sbar,sums,sumsb,v,w)

c     -------------------------------------------------------------------
c
c     local: A local search procedure to improve the solution obtained by
c            the GRASP construction phase
c
c     -------------------------------------------------------------------
c
c     Passed input scalars:
c
c          ne     - number of arcs in graph
c          n2e    - array dimension = 2*ne
c          nv     - number of nodes in graph
c
c     ------------------------------------------------------------------
      implicit   none
      integer    ne,n2e,nv

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          objf   - value of objective function
c
c     ------------------------------------------------------------------
      integer objf

c     ------------------------------------------------------------------
c
c     Passed input/output arrays:
c
c          edges    - 
c          first    - 
c          next
c          S
c          Sbar
c          sums
c          sumsb
c          v
c          w
c
c     ------------------------------------------------------------------
      integer    edge(n2e),first(nv),next(n2e),S(nv),
     +           Sbar(nv),sums(nv),sumsb(nv),
     +           v(n2e),w(ne)

c     ------------------------------------------------------------------
c
c     Passed input/output scalars:
c
c          nS
c          nSbar
c
c     -----------------------------------------------------------------
      integer    nS,nSbar

c     -----------------------------------------------------------------
c
c     Local scalars:
c
c          c      - do loop index
c          delta  - cost difference
c          j      - do loop index
c          optind - indicator of local optimility
c
c     ------------------------------------------------------------------
      integer c,delta,j,optind

c     ------------------------------------------------------------------
      optind=0
10    if (optind.eq.1) goto 70
           optind=1
c          -------------------------------------------------------------
c          Compute the min cardinality between S and Sbar.
c          -------------------------------------------------------------
           if (nSbar .gt. nS) then
                j=nS
           else
                j=nSbar
           endif

c          -------------------------------------------------------------
c          Starting from the first element of S0 and the first of Sbar0, 
c          the neighbors are the following.
c  
c          Let node be in S and u be in Sbar:
c
c          Type 1) S = S \ {node}; Sbar = Sbar U {node}
c
c          Type 2) S = S U {u}; Sbar = Sbar \ {u}
c
c          Even if node (u) remains in S (Sbar) (because nonimproving),
c          node (u) is shifted in the last position in S (Sbar) and the 
c          next element to be checked is always in the first position.
c
c          When one element is added to S (Sbar) it is added in the last 
c          position. 
c          ------------------------------------------------------------- 
           do 40 c=1,j
c               --------------------------------------------------------
c               Neighbor type 1 (on S)
c               --------------------------------------------------------

                call move(edge,delta,first,ne,n2e,next,nS,
     +                    nSbar,nv,S,Sbar,sums,sumsb,v,w)

	        if (delta.gt.0) then
	             objf=objf+delta
	             optind=0
	             goto 10
                endif

c               --------------------------------------------------------
c               Neighbor type 2 (on Sbar)
c               --------------------------------------------------------

                call move(edge,delta,first,ne,n2e,next,nSbar,
     +                    nS,nv,Sbar,S,sumsb,sums,v,w)

	        if (delta.gt.0) then
	            objf=objf+delta
	            optind=0
	            goto 10
                endif

40         continue  

           if (j .eq. nS) then
                j=nS+1
                do 50 c=j,nSbar
c                    ---------------------------------------------------
c                    Neighbor type 2 (on Sbar)
c                    ---------------------------------------------------
                     call move(edge,delta,first,ne,n2e,next,
     +                         nSbar,nS,nv,Sbar,S,sumsb,sums,v,w)

	             if (delta.gt.0) then
	                  objf=objf+delta
	                  optind=0
		          goto 10
                     endif
50              continue
           else
                j=nSbar+1
                do 60 c=j,nS
c                    ---------------------------------------------------
c                    Neighbor type 1 (on S)
c                    ---------------------------------------------------
                     call move(edge,delta,first,ne,n2e,next,
     +                         nS,nSbar,nv,S,Sbar,sums,sumsb,v,w)

	             if (delta.gt.0) then
	                  objf=objf+delta
	                  optind=0
		          goto 10
                     endif
60              continue
           endif 
           goto 10
70    return

c     ------------------------------------------------------------------
c     End of subroutine local
c     ------------------------------------------------------------------
      end
