      subroutine mkds (deg0,edge0,faselt,felt,first0,maxc,n2e,ne,nelt,
     +                 next0,nmelt,nmxelt,nv,nxtelt,v0,vertex,vtx1,
     +                 vtx2,w)
c     ------------------------------------------------------------------
c
c     mkds: Builds adjacency data structure for graph, computes degree.
c
c     ------------------------------------------------------------------
      implicit   none

c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          nv     - number of nodes in graph
c          ne     - number of arcs in graph
c          n2e    - array dimension = 2*ne
c          nmelt  -
c          nmxelt -
c
c     ------------------------------------------------------------------
      integer    ne,n2e,nv,nmelt,nmxelt

c     ------------------------------------------------------------------
c
c     Passed input arrays:
c    
c          vtx1  - node 1 of arc
c          vtx2  - node 2 of arc
c          w     - edge weights
c
c     ------------------------------------------------------------------
      integer   vtx1(ne),vtx2(ne),w(ne)

c     ------------------------------------------------------------------
c
c     Passed output arrays:
c    
c          deg0   - in-degree of vertex (original) 
c          edge0  - edge (original) 
c          felt   -
c          first0 - pointer to first vertex of in-adjacency 
c                   list (original)
c          next0  - pointer to next vertex of in-adjacency 
c                   list (original)
c          nxtelt -
c          vertex - existence of vertex in S U Sbar
c          v0     - vertex in in-adjacency list (original)
c
c     ------------------------------------------------------------------
      integer    deg0(nv),edge0(n2e),felt(nmxelt),
     +           first0(nv),next0(n2e),nxtelt(nmelt),
     +           vertex(nv),v0(n2e)

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c    
c          faselt -
c          maxc   - upper bound on cut value
c          nelt   -
c
c     ------------------------------------------------------------------
      integer faselt,maxc,nelt

c     ------------------------------------------------------------------
c
c     Local scalars:
c    
c          i      - do loop index
c          k      - counter
c          v1     - node 1
c          v2     - node 2
c    
c     -----------------------------------------------------------------
      integer    i,k,v1,v2

c     ------------------------------------------------------------------
c     Initialize pointers to list of adjacent vertices and degrees.
c     ------------------------------------------------------------------
      do 10 i=1,nv
          first0(i)=0
          deg0(i)=0
          vertex(i)=0
10    continue

      maxc=0
      do 20 i=1,ne
          next0(i)=i+1
	  maxc=maxc+iabs(w(i))
20    continue
      next0(ne)=0

c     ------------------------------------------------------------------
c     Create adjacency list.
c     ------------------------------------------------------------------
      k=1
      do 30 i=1,ne
c          -------------------------------------------------------------
c          Access directed edge = (v1,v2).
c          -------------------------------------------------------------
           v1=vtx1(i)
           v2=vtx2(i)

c          -------------------------------------------------------------
c          Add v1 to v2's adjacency list.
c          -------------------------------------------------------------
           v0(k)=v1
           edge0(k)=i
           next0(k)=first0(v2)
           first0(v2)=k

c          -------------------------------------------------------------
c          Update degree of v2
c          -------------------------------------------------------------
           deg0(v2)=deg0(v2)+1
           k=k+1

c          -------------------------------------------------------------
c          Add v2 to v1's adjacency list.
c          -------------------------------------------------------------
           v0(k)=v2
           edge0(k)=i
           next0(k)=first0(v1)
           first0(v1)=k

c          -------------------------------------------------------------
c          Update degree of v1
c          -------------------------------------------------------------
           deg0(v1)=deg0(v1)+1
           k=k+1
30    continue

c     ------------------------------------------------------------------
c     Initialize elite set data structure.
c     ------------------------------------------------------------------
      nelt=0
      do 40 i=1,nmxelt
           felt(i)=0
40    continue
      
      do 50 i=1,nmelt-1
           nxtelt(i)=i+1
50    continue
      nxtelt(nmelt)=0
      faselt=1

c     ------------------------------------------------------------------
      return
c
c     ------------------------------------------------------------------
c     End of subroutine mkds.
c     ------------------------------------------------------------------
      end
