      subroutine mkrcl(alpha,edge,first,maxc,ne,n2e,n2v,
     +                next,nrcl,nv,rcl,sumst,sumsbt,vertex,v,w)

c     ------------------------------------------------------------------
c
c     mkrcl: Build the restricted candidate list (RCL).
c
c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          alpha  - GRASP RCL parameter
c          maxc   - upper bound on maxcut cost
c          ne     - number of arcs in graph
c          n2e    - array dimension = 2*ne
c          n2v    - array dimension = 2*nv
c          nv     - number of nodes in graph
c
c     ------------------------------------------------------------------
      implicit   none
      real       alpha
      integer    maxc,ne,n2e,n2v,nv

c     ------------------------------------------------------------------
c
c     Passed input arrays:
c
c          edge   -
c          first  -
c          next   - 
c          vertex - existence of vertex in the reduced graph
c          v      -
c          w
c
c     ------------------------------------------------------------------
      integer    edge(n2e),first(nv),next(n2e),v(n2e),
     +           vertex(nv),w(ne)

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          nrcl   - number of elements in RCL
c
c     ------------------------------------------------------------------
      integer    nrcl

c     ------------------------------------------------------------------
c
c     Passed output array:
c
c          degs
c          degsb 
c          rcl   - Restricted Candidate List
c
c     ------------------------------------------------------------------
      integer    degs(nv),degsb(nv),rcl(n2v)

c     ------------------------------------------------------------------
c
c     Working arrays:
c
c          sumst
c          sumsbt
c
c     ------------------------------------------------------------------
      integer    sumst(nv),sumsbt(nv)

c     ------------------------------------------------------------------
c
c     Local scalars:
c
c          coff
c          e     - arc
c          maxs
c          maxsb
c          maxw
c          mins
c          minsb
c          minw
c          node  - vertex
c          u     - vertex
c
c     -------------------------------------------------------------------
      integer coff,e,maxs,maxsb,maxw,mins,minsb,minw,node,u

c     -------------------------------------------------------------------
      do 10 node=1,nv
         sumst(node)=0
         sumsbt(node)=0
10    continue

      mins=maxc
      minsb=maxc
      minw=maxc

      maxs=-maxc
      maxsb=-maxc
      maxw=-maxc

c     ------------------------------------------------------------------
c     Find the maximum and minimum values.
c     ------------------------------------------------------------------
      do 20 node=1,nv
c          -------------------------------------------------------------
c          Consider only vertices v belonging neither to S nor to Sbar.
c          -------------------------------------------------------------
           if (vertex(node).eq.1 .or. vertex(node).eq.2) go to 20 
           e=first(node)
30         if (e .eq. 0) go to 40
                u=v(e)

c               --------------------------------------------------------
c               Consider only vertices u belonging either to S or to 
c               Sbar.
c               --------------------------------------------------------
                if (vertex(u) .eq. 0) then
                     e=next(e)
                     go to 30
                endif

c               --------------------------------------------------------
c               Check if u belongs to S.
c               --------------------------------------------------------
                if (vertex(u) .eq. 1) then
                     sumst(node)=sumst(node)+w(edge(e))
                endif

c               --------------------------------------------------------
c               Check if u belongs to Sbar.
c               --------------------------------------------------------
                if (vertex(u) .eq. 2) then
                     sumsbt(node)=sumsbt(node)+w(edge(e))
                endif

c               --------------------------------------------------------
c               Consider the next adjacent arc to node.
c               --------------------------------------------------------
                e=next(e)
                go to 30
 
c          -------------------------------------------------------------
c          Update the min sum over S and the max sum over S.
c          -------------------------------------------------------------
40         mins = min (mins,sumst(node))
           maxs = max (maxs,sumst(node))

c          -------------------------------------------------------------
c          Update the min sum over Sbar and the max sum over Sbar.
c          -------------------------------------------------------------
           minsb = min (minsb,sumsbt(node))
           maxsb = max (maxsb,sumsbt(node))
20    continue

c     ------------------------------------------------------------------
c     Find the maximum and minimum values.
c     ------------------------------------------------------------------
      minw = min (mins,minsb)

      maxw = max (maxs,maxsb)

c     ------------------------------------------------------------------
c     Compute cut off.
c     ------------------------------------------------------------------
      coff=minw+ifix(alpha*(maxw-minw))

c     ------------------------------------------------------------------
c     Put vertex node into RCL if sums(node) or sumsb(node) >= cut off
c     ------------------------------------------------------------------
      nrcl=0
      do 50 node=1,nv
c          -------------------------------------------------------------
c          Consider only vertices node belonging neither to S nor to 
c          Sbar.
c          -------------------------------------------------------------
           if (vertex(node) .eq. 0) then
                if (sumst(node).ge.coff) then
                     nrcl=nrcl+1
                     rcl(nrcl)=-node
                endif
                if (sumsbt(node).ge.coff) then
                     nrcl=nrcl+1
                     rcl(nrcl)=node
                endif
      endif
50    continue     

c     ------------------------------------------------------------------
      return
c
c     ------------------------------------------------------------------
c     End of subroutine mkrcl.
c     ------------------------------------------------------------------
      end
