      subroutine pickv(edge,first,
     +                ne,n2e,n2v,next,node,nrcl,nS,
     +                nSbar,nv,
     +                nvleft,objf,rcl,S,Sbar,seed,v,vertex,w)
c     ------------------------------------------------------------------
c
c     pickv: Pick a vertex at random from elements of Restricted
c            Candidate List (RCL). 
c
c     ------------------------------------------------------------------
c
c     Passed input scalars:
c
c          ne     - number of arcs in graph
c          n2e    - array dimension = 2*ne
c          n2v    - array dimension = 2*nv
c          nv     - number of nodes in graph
c          seed   - seed for pseudo random number generator
c
c     ------------------------------------------------------------------
      implicit   none
      integer ne,n2e,n2v,nv,seed

c     ------------------------------------------------------------------
c
c     Passed input/output scalar:
c
c          objf   - 
c          nrcl   - number of vertices currently in RCL
c          nS
c          nSbar
c          nvleft
c
c     ------------------------------------------------------------------
      integer   objf,nrcl,nS,nSbar,nvleft

c     ------------------------------------------------------------------
c
c     Passed input array:
c
c          edge
c          first
c          rcl    - Restricted Candidate List
c          v
c          vertex
c          w
c
c     ------------------------------------------------------------------
      integer   edge(n2e),first(nv),
     +          next(n2e),rcl(n2v),v(n2e),vertex(nv),
     +          w(ne)

c     ------------------------------------------------------------------ 
c
c     Passed input/output array:
c
c          S
c          Sbar 
c
c     ------------------------------------------------------------------
      integer   S(nv),Sbar(nv)
c     ------------------------------------------------------------------
c
c     Local scalars and functions:
c
c          e      - arc
c          node   - vertex
c          nselct - pointer to vertex
c          randp  - random number generator
c          u      - vertex
c          xrand  - random number
c
c     ------------------------------------------------------------------
      integer   e,node,nselct,u
      real      randp,xrand

c     ------------------------------------------------------------------
c     Pick node at random.
c     ------------------------------------------------------------------
      xrand=randp(seed)
      nselct=1+seed/(2147483647/nrcl)
      node=rcl(nselct)

c     ------------------------------------------------------------------
c     Put node in cut set.
c     ------------------------------------------------------------------
      if (node.lt.0) then
         nSbar=nSbar+1
         Sbar(nSbar)=-node
         vertex(-node)=2
	 node=-node
      else
         nS=nS+1
         S(nS)=node
         vertex(node)=1
      endif

c     ------------------------------------------------------------------
c     Check if node has been added to S. If yes, all the arcs adjacent  
c     to node belonging to Sbar are added to L.
c     ------------------------------------------------------------------
      if (vertex(node).eq.1 )then
c          -------------------------------------------------------------
c          Node has been added to S.
c          -------------------------------------------------------------
           e=first(node) 
10         if (e.eq.0) go to 20
                u=v(e)
                if (vertex(u).eq.2) then
	             objf=objf+w(edge(e))
                endif
                e=next(e)
                go to 10
      else 
c          -------------------------------------------------------------
c          Node has been added to Sbar.
c          -------------------------------------------------------------
           e=first(node) 
30         if (e.eq.0) go to 20
                u=v(e)
                if (vertex(u).eq.1) then
	             objf=objf+w(edge(e))
                endif
                e=next(e)
                go to 30
      endif
20    nvleft=nvleft-1
c     ------------------------------------------------------------------
      return
c
c     ------------------------------------------------------------------
c     End of subroutine pickv.
c     ------------------------------------------------------------------
      end
