      subroutine vns(edge,first,ne,n2e,next,nS,
     +               nSbar,nv,objf,S,Sbar,sums,sumsb,
     +               v,w,x,xn,nindex,kmax,seedn,tvnS,
     +               tvnSb,look4,btime,vtx1,vtx2)

c     -------------------------------------------------------------------
c
c     vns: A variable neighborhood search procedure to improve the 
c          solution obtained by the GRASP construction phase
c
c     -------------------------------------------------------------------
c
c     Passed input scalars:
c
c          ne     - number of arcs in graph
c          n2e    - array dimension = 2*ne
c          nv     - number of nodes in graph
c          kmax   - max K for VNS
c          seedn  - seed for k-neighbor generator 
c          look4  - target value
c
c     ------------------------------------------------------------------
      implicit   none
      integer    ne,n2e,nv,kmax,seedn,ntvnS,ntvnSb,look4

c     ------------------------------------------------------------------
c
c     Passed output scalars:
c
c          objf   - value of objective function
c          pobjf  - value of previous objective function
c
c     ------------------------------------------------------------------
      integer objf,pobjf

c     ------------------------------------------------------------------
c
c     Passed input/output arrays:
c
c          edges    - 
c          first    - 
c          next
c          S
c          Sbar
c          sums
c          sumsb
c          v
c          w
c
c     ------------------------------------------------------------------
      integer    edge(n2e),first(nv),next(n2e),S(nv),tvnS(nv),
     +           tvnSb(nv),vtx1(ne),vtx2(ne),
     +           Sbar(nv),sums(nv),sumsb(nv),
     +           v(n2e),w(ne),x(nv),xn(nv),nindex(nv)

c     ------------------------------------------------------------------
c
c     Passed input/output scalars:
c
c          nS
c          nSbar
c
c     -----------------------------------------------------------------
      integer    nS,nSbar

c     -----------------------------------------------------------------
c
c     Local scalars:
c
c          i      - do loop index
c          k      - do loop index
c
c     ------------------------------------------------------------------
      integer i,j,k,ptr
      real etime,cput(2),btime,ttime

c     ------------------------------------------------------------------
c     Save objf of current solution x in variable pobjf and
c     current node partition in 0-1 array x.
c     ------------------------------------------------------------------
c     print *,'vns: nv = ',nv
      call cpi4(nS,S,tvnS)
      ntvnS=nS
1     pobjf=objf
c     print *,'VNS: start/restart: pobjf= ',pobjf
      do 10 i=1,nv
           x(i)=0
10    continue
      nS=ntvnS
      do 20 i=1,nS
           S(i)=tvnS(i)
           x(S(i))=1
20    continue
c     print *,'x: ',(x(i),i=1,nv)
      print *,'objf(x): ',objf

c     ------------------------------------------------------------------
c     Start/restart VNS loop.
c     ------------------------------------------------------------------
      k=1
30    if (k.gt.kmax) goto 40
c          -------------------------------------------------------------
c          Generate k-neighbor of current solution in 0-1 array xn. 
c          -------------------------------------------------------------
c          print *,'VNS: generate neighbor: k= ',k
           call genkn(nv,k,x,nindex,xn,seedn)

c          -------------------------------------------------------------
c          Jump:  move current solution to k-neighbor.
c          -------------------------------------------------------------
c          print *,'xn: ',(xn(i),i=1,nv)
           objf=0
           do 45 i=1,nv
                sums(i)=0
                sumsb(i)=0
45         continue
           ntvnS=0
           ntvnSb=0
           do 50 i=1,nv
                if (xn(i).eq.1) then
                     ntvnS=ntvnS+1
                     tvnS(ntvnS)=i
                     ptr=first(i)
46                   if (ptr.eq.0) goto 47
                          j=v(ptr)
                          sums(j)=sums(j)+w(edge(ptr))
                          if (xn(j).eq.0) then
                               objf=objf+w(edge(ptr))
                          endif
                          ptr=next(ptr)
                          goto 46
47                   continue
                else
                     ntvnSb=ntvnSb+1
                     tvnSb(ntvnSb)=i
                     ptr=first(i)
56                   if (ptr.eq.0) goto 57
                          j=v(ptr)
c                         print *,'adj v j= ',j
                          sumsb(j)=sumsb(j)+w(edge(ptr))
                          ptr=next(ptr)
                          goto 56
57                   continue
                endif
50         continue
           print *,'b4 local search objf(xn): ',objf
c          -------------------------------------------------------------
c          Do local search on k-neighbor.
c          -------------------------------------------------------------
           call local(edge,first,ne,n2e,next,ntvnS,
     +                ntvnSb,nv,objf,tvnS,tvnSb,sums,sumsb,v,w)

           print *,'VNS, after local search: k= ',k,
     +             ' local optimum of neighbor (prev/new) = ',
     +              pobjf,objf
c          -------------------------------------------------------------
c          VNS test: if improvement is achieved, then restart VNS with
c                    k=1.  Otherwise, increment k and continue.
c          -------------------------------------------------------------
           if (objf.gt.pobjf) then
                if (objf.ge.look4) then
                     call cpi4(ntvnS,tvnS,S)
                     call cpi4(ntvnSb,tvnSb,Sbar)
		     nS=ntvnS
		     nSbar=ntvnSb
		     return
		endif
		goto 1
           endif

c          print *,'VNS: increment k= ',k+1
           k=k+1
           objf=pobjf
	   ttime=etime(cput)
           print *, ' total time = ',cput(1)-btime
           goto 30
40    continue

      
      print *,' b4 final VNS check, objf = ',objf
c     crint *,'b4 do 945: x= ',(x(i),i=1,nv)
      do 945 i=1,nv
           sums(i)=0
           sumsb(i)=0
945   continue
      do 950 i=1,nv
           if (x(i).eq.1) then
                 ptr=first(i)
946              if (ptr.eq.0) goto 947 
                      j=v(ptr)
                      sums(j)=sums(j)+w(edge(ptr))
                      ptr=next(ptr)
                      goto 946
947             continue
           else
                ptr=first(i)
956             if (ptr.eq.0) goto 957
                     j=v(ptr)
                     sumsb(j)=sumsb(j)+w(edge(ptr))
                     ptr=next(ptr)
                     goto 956
957             continue
           endif
950   continue

      return
c     call cpi4(ntvnS,tvnS,S)
c     call cpi4(ntvnSb,tvnSb,Sbar)
c     nS=ntvnS
c     nSbar=ntvnSb
      print *,'after 950: objf = ',objf

      do 9100 i=1,nv
	      x(i)=0
9100  continue
      do 9101 i=1,nS
              x(S(i))=1
9101  continue
      objf=0
      do 9102 i=1,ne
           if (x(vtx1(i))+x(vtx2(i)) .eq. 1) objf=objf+w(i)
9102  continue
      print *,' objf recomputed in VNS: ',objf

      return
                
c     ------------------------------------------------------------------
c     End of subroutine vns
c     ------------------------------------------------------------------
      end
